var Container = /** @class */ (function () {
    function Container(parentContainer) {
        this._map = new Map();
        this._parentContainer = parentContainer;
    }
    Container.prototype.get = function (key) {
        var isMultiInject = Array.isArray(key);
        if (Array.isArray(key)) {
            if (key.length > 0) {
                key = key[0];
            }
            else {
                throw new Error("Invalid multi inject of `" + key.toString() + "`.");
            }
        }
        var entries = this._map.get(key);
        if (isMultiInject && (!entries || entries.length === 0)) {
            return [];
        }
        if (!entries || entries.length === 0) {
            if (this._parentContainer) {
                return this._parentContainer.get(key);
            }
            throw new Error("Missing binding of `" + key.toString() + "`.");
        }
        var resolvedEntries = [];
        for (var _i = 0, entries_1 = entries; _i < entries_1.length; _i++) {
            var entry = entries_1[_i];
            if (entry.kind === 'newable' && !entry.value) {
                entry.value = this.resolve(entry.newable, entry.params);
            }
            if (entry.kind === 'factory' && !entry.value) {
                entry.value = entry.factory(this);
            }
            if (isMultiInject) {
                resolvedEntries.push(entry.value);
            }
            else {
                return entry.value;
            }
        }
        return resolvedEntries;
    };
    Container.prototype.bind = function (serviceIdentifier, constructor, params) {
        if (params === void 0) { params = []; }
        var entries = this._map.get(serviceIdentifier) || [];
        entries.push({
            kind: 'newable',
            newable: constructor,
            params: params
        });
        this._map.set(serviceIdentifier, entries);
    };
    Container.prototype.bindToConstant = function (serviceIdentifier, constant) {
        var entries = this._map.get(serviceIdentifier) || [];
        entries.push({
            kind: 'constant',
            value: constant
        });
        this._map.set(serviceIdentifier, entries);
    };
    /**
     * Bind the implementations of the class to the class.
     * @param constructor
     * @param params
     * @param serviceIdentifiers
     */
    Container.prototype.bindToImplements = function (constructor, params, serviceIdentifiers) {
        if (params === void 0) { params = []; }
        if (serviceIdentifiers === void 0) { serviceIdentifiers = []; }
        var entry = {
            kind: 'newable',
            newable: constructor,
            params: params
        };
        for (var _i = 0, serviceIdentifiers_1 = serviceIdentifiers; _i < serviceIdentifiers_1.length; _i++) {
            var serviceIdentifier = serviceIdentifiers_1[_i];
            var entries = this._map.get(serviceIdentifier) || [];
            entries.push(entry);
            this._map.set(serviceIdentifier, entries);
        }
    };
    Container.prototype.bindToFactory = function (serviceIdentifier, factory) {
        var entries = this._map.get(serviceIdentifier) || [];
        entries.push({
            kind: 'factory',
            factory: factory
        });
        this._map.set(serviceIdentifier, entries);
    };
    Container.prototype.resolve = function (service, params) {
        if (params === void 0) { params = []; }
        var args = [];
        for (var _i = 0, params_1 = params; _i < params_1.length; _i++) {
            var val = params_1[_i];
            args.push(this.get(val));
        }
        return new (service.bind.apply(service, [void 0].concat(args)))();
    };
    Container.prototype.resolveFactory = function (service, params) {
        if (params === void 0) { params = []; }
        var args = [];
        for (var _i = 0, params_2 = params; _i < params_2.length; _i++) {
            var val = params_2[_i];
            args.push(this.get(val));
        }
        return service.bind.apply(service, [undefined].concat(args));
    };
    return Container;
}());
export { Container };
//# sourceMappingURL=api.js.map